;RS-232 Receiver
 INCLUDE "C:\MPIC\P16F84.INC"


;Bit definitions

#define RX_Pin PORTA,4   ;RA4 is Input Pin
#define _FChar SerialStatus,7
#define _FrameErr SerialStatus,5
#define _RcvProgress SerialStatus,2
#define _RcvOver SerialStatus,3
 
;Registers

SerialStatus equ 0C
Bitcount equ 0D
RXREG equ 0E
SaveWReg equ 0F
SaveStatus equ 10

 __CONFIG _CP_OFF & _WDT_OFF & _XT_OSC

 ORG 0
 goto InitPorts
 ORG 4
 goto Interrupt

InitPorts:
 clrf SerialStatus
 bsf _RcvOver
 bsf STATUS, RP0
 bsf RX_Pin	;configure RA4 for input
 bcf PORTA,3
 clrf TRISB	;configure PORTB as output
 bcf STATUS, RP0
 movlw b'10101010'
 movwf RXREG
 bsf PORTA,3

NextChar
 ;bcf STATUS,RP0
 comf RXREG,w
 movwf PORTB	;Prints default pattern on reset and character pattern upon receipt
 

StartReception:
 bcf STATUS, RP0
 ;bsf _RcvOver
 movlw 8	;number of bits
 movwf Bitcount
 clrf RXREG
 
 bcf _FrameErr
 
;
 clrf TMR0
 clrwdt
 bsf STATUS,RP0
 movlw 07
 movwf OPTION_REG
 bcf STATUS,RP0
 clrf TMR0
 bsf STATUS,RP0
 movlw 0F
 movwf OPTION_REG
 clrwdt
 
 movlw b'00111000'	;prescaler 000 assigned to wdt. TMR0 incr with every execute cycle
 movwf OPTION_REG		;EXT clock, increments on falling edge
 bcf STATUS,RP0
;
 movlw 0FF
 movwf TMR0	;0xFF=256 loaded in TMR0. Overflow with first falling edge.

 
 bcf INTCON,T0IF
 bsf INTCON,T0IE
 bsf INTCON, GIE 

 
 btfss _RcvOver
 goto NextChar
 comf RXREG,w
 movfw PORTB  
 goto $-4	;_RcvOver is cleared when all 8 bits are received
 
 goto NextChar


Interrupt:
 bcf _FChar
;Save Status and W Regs
 
 movwf SaveWReg
 swapf STATUS,w
 movwf SaveStatus
;
 btfsc _RcvProgress
 goto _RcvNextBit
 goto _StartBitdet
;
_StartBitdet
 bcf STATUS, RP0
 btfsc RX_Pin	;verify start bit
 goto _FalseStartBit
 bsf _RcvProgress
 clrf TMR0
 ;Switch Prescaler Assignment
 clrf TMR0
 clrwdt
 bsf STATUS,RP0
 movlw 07
 movwf OPTION_REG
 bcf STATUS,RP0
 clrf TMR0
 bsf STATUS,RP0
 movlw 0F
 movwf OPTION_REG
 clrwdt

 movlw b'00000000'
 movwf OPTION_REG
 bcf STATUS, RP0
 movlw d'152'	;Initial TMR0=152 with a prescaler 1:2
 movwf TMR0	;produces a TMR0 delay=208ms (4800 bps)

 goto RestoreIntStatus
_FalseStartBit
 movlw 0FF
 movwf TMR0	;Reload TMR0 with 0xFF for start bit detection
 goto RestoreIntStatus

_RcvNextBit

 clrwdt
 bsf STATUS,RP0
 movlw 07
 movwf OPTION_REG
 bcf STATUS,RP0
 clrf TMR0
 clrwdt
 bsf STATUS,RP0
 movlw 07
 movwf OPTION_REG
 bcf STATUS, RP0
 clrf TMR0
 bsf STATUS,RP0
 movlw 0F
 movwf OPTION_REG
 clrwdt
;
 movlw b'00000000'
 movwf OPTION_REG
 bcf STATUS, RP0
 movlw d'152'	;Initial TMR0=152 with a prescaler 1:2
 movwf TMR0	;produces a TMR0 delay=208ms (4800 bps)
;
 bsf STATUS,C
 btfss RX_Pin;Read Pin RA4
 bcf STATUS,C
 rrf RXREG
 decfsz Bitcount
 goto RestoreIntStatus
 btfss RX_Pin	;read Stop bit
 bsf _FrameErr
 bcf _RcvProgress
 bcf _RcvOver
 
 goto RestoreIntStatus


RestoreIntStatus
 swapf SaveStatus,w
 movwf STATUS
 swapf SaveWReg
 swapf SaveWReg,w 
 bcf INTCON, T0IF
 retfie

 end
 

