;                         
;
; ###############################################################################
;	filename:		Discriptor.ASM
;				USB Descriptors and other user modified code
;
; Implements the descriptor lookup tables and other user modified code.  
; Currently this consists of SetConfiguration, which is called when the
; host sends a Set Configuration command.  SetConfiguration gives meaning
; to the configuration.
;
; ###############################################################################
;
;    	Author:			Dan Butler
;    	Company:		Microchip Technology Inc
;
;	Revision:		1.20
;	Date:			26 April 2001
;	Assembled using		MPASM 2.30.07
;	Change log:
;	24 August 2000  DZB  Changed descriptor tables to 16 bit pointers
;
;################################################################################
;
;	include files:
;		P16C745.inc	Rev 1.00
;		usb_defs.inc	Rev 1.10
;
;################################################################################
#include <p16C745.inc>
#include "usb_defs.inc"

descriptors	code
	global	Config_desc_index
	global	Descriptions
	global	string_index
	global	DeviceDescriptor
	global	ReportDescriptor
	global	ReportDescriptorLen
	global	HID_Descriptor
	global	String0
	global	String0_end

	extern	EP0_start
	extern 	temp	; temp var used in get config index
	extern	temp2	; another temp, in bank2

; ******************************************************************
; Given a configuration descriptor index, returns the beginning address
; of the descriptor within the descriptions table
; ******************************************************************
Config_desc_index
	movwf	temp
	movlw	HIGH CDI_start
	movwf	PCLATH
	movlw	low  CDI_start
	addwf	temp,w
	btfsc	STATUS,C
	incf	PCLATH,f
	movwf	PCL
CDI_start	; this table calculates the offsets for each configuration
	retlw	low  Config1		; descriptor from the beginning
	retlw	high Config1		; of the table, effectively
	; more configurations can be added here
	; retlw	low Config2
	; retlw high Config2
	; etc....

; ******************************************************************
; This table is polled by the host immediately after USB Reset has been released.
; This table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application DEPENDENT. Modify these to meet
; your specifications.
; the offset is passed in P0 and P1 (P0 is low order byte).
; ******************************************************************
Descriptions
	banksel	EP0_start
	movf	EP0_start+1,w
	movwf	PCLATH
	movf	EP0_start,w
	movwf	PCL

DeviceDescriptor
StartDevDescr
	retlw	.18		; bLength	Length of this descriptor
	retlw	1		; bDescType 	This is a DEVICE descriptor
	retlw	0;.10		; bcdUSB	USB revision 1.10 (low byte)
	retlw	1		; high byte
	retlw	0		; bDeviceClass	zero means each interface operates independently
	retlw	0		; bDeviceSubClass
	retlw	0		; bDeviceProtocol
	retlw	0x08		; bMaxPacketSize0 - inited in UsbInit()
	retlw	0x25		; idVendor	0x0925 is LakeView Research ID
	retlw	0x09		; high order byte
	retlw	0x34		; idProduct
	retlw	0x12
	retlw	0x01		; bcdDevice
	retlw	0x00
	retlw	0;0x01		; iManufacturer
	retlw	0;0x02		; iProduct
	; somehow, changing iSerialNumber will cause windows to display
	; "New Hardware found" dialog. The device will function normally,
	; however, USBCheck will NOT start. From CATC we can see that
	; the string is correctly transmitted.
	retlw	0x0		; iSerialNumber - 3
	retlw	NUM_CONFIGURATIONS ; bNumConfigurations

; ******************************************************************
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application DEPENDENT. 
; Modify these to meet your specifications.
; ******************************************************************
Config1
	retlw	9		; bLength	Length of this descriptor
	retlw	2		; bDescType	2=CONFIGURATION

	retlw	EndConfig1 - Config1
	retlw	0
	retlw	1		; bNumInterfaces	Number of interfaces

	retlw	0x01		; bConfigValue	Configuration Value
	retlw	0;0x04		; iConfig	String Index for this config = #01
	retlw	0xA0		; bmAttributes	attributes - bus powered
	retlw	0x32		; MaxPower	26 mA from the bus.
Interface1
	retlw	9		; length of descriptor
	retlw	INTERFACE
	retlw	0		; number of interface, 0 based array
	retlw	0		; alternate setting
	retlw	1		; number of endpoints used in this interface
	retlw	3		; interface class - assigned by the USB
	retlw	0		; boot device
	retlw	0		; interface protocol - mouse
	retlw 	0		; index to string descriptor that describes this interface
HID_Descriptor
        retlw	09h		; descriptor size (9 bytes)
        retlw	21h		; descriptor type (HID)
        retlw	00h
	 retlw		01h		; HID class release number (1.00)
        retlw	00h		; Localized country code (none)
        retlw	01h		; # of HID class descriptor to follow (1)
        retlw	22h		; Report descriptor type (HID)
        retlw	(end_ReportDescriptor - ReportDescriptor)
        retlw	00h
Endpoint1
	retlw	7		; length of descriptor
	retlw	ENDPOINT
	retlw	0x81		; EP1, In
	retlw	3		; Interrupt
	retlw	4		; max packet size (4 bytes) low order byte
	retlw	0		; max packet size (4 bytes) high order byte
	retlw	0x0a		; polling interval (10ms)
EndConfig1

ReportDescriptorLen
	retlw	low  (end_ReportDescriptor-ReportDescriptor)

ReportDescriptor
        retlw	0x06		; usage page (Vendor Defined)
	retlw	0xA0
        retlw	0xFF
	retlw	0x09		; usage (Vendor Defined)
        retlw	0x01
	retlw	0xA1		; collection (application)
        retlw	0x01
	retlw	0x09		; usage (Vendor Defined)
        retlw	0x02
	retlw	0xA1		; collection (linked)
        retlw	0x00
	retlw	0x06		; usage page (buttons)
        retlw	0xa1
	retlw	0xFF
	retlw	0x09		; usage vendor defined
        retlw	0x03
	retlw	0x09		; usage vendor defined
        retlw	0x04
	retlw	0x15		; logical minimum (-128)
        retlw	0x80
	retlw	0x25		; logical maximum (127)
        retlw	0x7F
	retlw	0x35		; Physical Minimum (0)
        retlw	0x00
	retlw	0x45		; Physical Maximum (255)
        retlw	0xFF
;	retlw	0x66		; Unit (None (2 bytes))
;       retlw	0x00
;	retlw	0x00
        retlw	0x75		; report size (8) (bits)
	retlw	0x08
        retlw	0x95		; report count (2) (fields)
	retlw	0x02
        retlw	0x81		; input (Data, Variable, Absolute)
	retlw	0x02

        retlw	0x09		; usage (Vendor Defined)
	retlw	0x05
        retlw	0x09		; useage (Vendor Defined)
	retlw	0x06
	retlw	0x15		; logical minimum (-128)
	retlw	0x80
        retlw	0x25		; logical maximum (127)
	retlw	0x7F
        retlw	0x35		; Physical Minimum (0)
	retlw	0x00
        retlw	0x45		; Physical Maximum (255)
	retlw	0xFF
;	retlw	0x66		; Unit (None (2 bytes))
;       retlw	0x00
;	retlw	0x00
        retlw	0x75		; report size (8) (bits)
	retlw	0x08
        retlw	0x95		; report Count (2) (fields)
	retlw	0x02
        retlw	0x91		; Output (Data, Variable, Absolute)
	retlw	0x02
        retlw	0xC0		; end collection, end collection
	retlw	0xC0

end_ReportDescriptor


; ******************************************************************
; Given a configuration descriptor index, returns the beginning address
; of the descriptor within the descriptions table
; ******************************************************************
string_index			; langid in W reg, string offset in EP0_start
	movwf	temp
	bcf	STATUS,C
	rlf	temp, f
	pagesel	langid_index
	call	langid_index
	movwf	temp2
	incf	temp, f
	pagesel	langid_index
	call	langid_index
	movwf	temp
	
	movf	temp, w
	movwf	PCLATH
	movf	temp2,w
	addwf	EP0_start+1,w
	btfsc	STATUS,C
	incf	PCLATH, f
	movwf	PCL

langid_index
	movlw	high langids
	movwf	PCLATH
	movlw	low langids
	addwf	temp, w
	btfsc	STATUS,C
	incf	PCLATH,f
	movwf	PCL

langids
	retlw	low lang_1
	retlw	high lang_1
	retlw	low lang_2	; string indexes of different languages
	retlw	high lang_2

lang_1				; english
	retlw	low  String0	; LangIDs
	retlw	high String0
	retlw	low  String1_l1	
	retlw	high String1_l1
	retlw	low  String2_l1
	retlw	high String2_l1
	retlw	low  String3_l1
	retlw	high String3_l1
	retlw	low  String4_l1
	retlw	high String4_l1
	retlw	low  String5_l1
	retlw	high String5_l1

lang_2
	retlw	low  String0	; also point to LangID
	retlw	high String0
	retlw	low  String1_l2
	retlw	high String1_l2
	retlw	low  String2_l2
	retlw	high String2_l2
	retlw	low  String3_l2
	retlw	high String3_l2
	retlw	low  String4_l2
	retlw	high String4_l2
	retlw	low  String5_l2
	retlw	high String5_l2

String0
	retlw	low (String1_l1 - String0) ; length of string 
	retlw	0x03		; descriptor type 3?
	retlw	0x09		; language ID (as defined by MS 0x0409)
	retlw	0x04
	retlw	0x04		; some other language ID for testing
	retlw	0x08
String0_end
String1_l1
	retlw	String2_l1-String1_l1 ; length of string
	retlw	0x03		; string descriptor type 3
	retlw	'M'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'r'
	retlw	0x00
	retlw	'o'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'h'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'p'
	retlw	0x00
String2_l1
	retlw	String3_l1-String2_l1
	retlw	0x03
	retlw	'P'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'6'
	retlw	0x00
	retlw	'C'
	retlw	0x00
	retlw	'7'
	retlw	0x00
	retlw	'6'
	retlw	0x00
	retlw	'5'
	retlw	0x00
	retlw 	' '
	retlw	0x00
	retlw	'U'
	retlw	0x00
	retlw	'S'
	retlw	0x00
	retlw	'B'
	retlw	0x00
	retlw	' '
	retlw	0x00
	retlw	'M'
	retlw	0x00
	retlw	'o'
	retlw	0x00
	retlw	'u'
	retlw	0x00
	retlw	's'
	retlw	0x00
	retlw	'e'
	retlw	0x00
String3_l1
	retlw	String4_l1-String3_l1
	retlw	0x03
	retlw	'V'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'.'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'1'
	retlw	0x00
String4_l1
	retlw	String5_l1-String4_l1
	retlw	0x03
	retlw	'C'
	retlw	0x00
	retlw	'f'
	retlw	0x00
	retlw 	'g'
	retlw	0x00
	retlw 	'1'
	retlw	0x00
String5_l1
	retlw	String6_l1-String5_l1
	retlw	0x03
	retlw	'E'
	retlw	0x00
	retlw	'P'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'0'
	retlw	0x00
	retlw	'I'
	retlw	0x00
	retlw	'n'
	retlw	0x00
String6_l1

String1_l2				; lang 2, chinese. String can be totally different than english
	retlw	String2_l2-String1_l2	; length of string
	retlw	0x03			; string descriptor type 3
	retlw	'M'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'r'
	retlw	0x00
	retlw	'o'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'h'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'p'
	retlw	0x00
String2_l2
	retlw	String3_l2-String2_l2
	retlw	0x03
	retlw	'P'
	retlw	0x00
	retlw	'i'
	retlw	0x00
	retlw	'c'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'6'
	retlw	0x00
	retlw	'C'
	retlw	0x00
	retlw	'7'
	retlw	0x00
	retlw	'4'
	retlw	0x00
	retlw	'5'
	retlw	0x00
	retlw 	' '
	retlw	0x00
	retlw	'U'
	retlw	0x00
	retlw	'S'
	retlw	0x00
	retlw	'B'
	retlw	0x00
	retlw	' '
	retlw	0x00
	retlw	'D'
	retlw	0x00
	retlw	'e'
	retlw	0x00
	retlw	'm'
	retlw	0x00
	retlw	'o'
	retlw	0x00
String3_l2
	retlw	String4_l2-String3_l2
	retlw	0x03
	retlw	'B'
	retlw	0x00
	retlw	'e'
	retlw	0x00
	retlw	't'
	retlw	0x00
	retlw	'a'
	retlw	0x00
String4_l2
	retlw	String5_l2-String4_l2
	retlw	0x03
	retlw	'C'
	retlw	0x00
	retlw	'f'
	retlw	0x00
	retlw 	'g'
	retlw	0x00
	retlw 	'1'
	retlw	0x00
String5_l2
	retlw	String6_l2-String5_l2
	retlw	0x03
	retlw	'E'
	retlw	0x00
	retlw	'P'
	retlw	0x00
	retlw	'1'
	retlw	0x00
	retlw	'0'
	retlw	0x00
	retlw	'I'
	retlw	0x00
	retlw	'n'
	retlw	0x00
String6_l2

	end

