;                            Software License Agreement
;
; The software supplied herewith by Microchip Technology Incorporated (the "Company")
; for its PICmicro Microcontroller is intended and supplied to you, the Companys
; customer, for use solely and exclusively on Microchip PICmicro Microcontroller
; products.
;
; The software is owned by the Company and/or its supplier, and is protected under
; applicable copyright laws. All rights are reserved. Any use in violation of the
; foregoing restrictions may subject the user to criminal sanctions under applicable
; laws, as well as to civil liability for the breach of the terms and conditions of
; this license.
;
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, WHETHER EXPRESS,
; IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
; COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;
; ###############################################################################
;	filename:		USB_MAIN.ASM
;				Sample mainline program
;
; Microchip's development environment including Assembler, Simulator and Linker is
; available free from the Microchip website www.microchip.com.  From there follow
; the Development Tools tab and download MPLAB-IDE.  MPLAB is a 10Mb download.  You
; can also request a CD at http://www.microchip.com/10/lit/request/techcard/index.htm.
; Fill out the form and mark the box "2001 Technical CD ROM" at the bottom of the form.
;
; This file implements a basic interrupt service routine and shows how the
; USB interrupt would be serviced, and also how InitUSB, GetUSB and PutUSB
; should be called.  It may be useful as a reference, or as a starting point 
; from which to build an application.  
;
; This version implements the functionality of the Jan Axelson demo which accepts
; two numbers from the host, increments each and sends the back.  An application
; running on the host sends the numbers and displays the returned values.
;
; 
;###############################################################################
;
;    	Author:			Dan Butler
;    	Company:		Microchip Technology Inc
;
;	Revision:		1.20
;	Date:			17 May 2001
;	Assembled using		MPASM 2.30.07
;	Configuration Bits:	H4 Oscillator, WDT Off, Power up timer off
;	Revision History:
;	02 May 2001	DZB	Implemented SHOW_ENUM_STATUS to show enumeration
;				status on the PORTB LEDs: 0- Powered, 1- Default,
;				2- addressed, 3- configured, 4- sleep, 
;				5- EP0 Activity, 6- EP1 Activity, 7- EP2 Activity
;
;################################################################################
;
;	include files:
;		P16C745.inc	Rev 1.00
;		usb_defs.inc	Rev 1.00
;
;################################################################################
#include <p16c745.inc>
#include "usb_defs.inc"

	__CONFIG	_H4_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF

unbanked	udata_shr
W_save		res	1

bank0	udata
Status_save	res	1	; registers for saving context
PCLATH_save	res	1
FSR_save	res	1

inner		res	1	; delay loop counters
middle		res	1
outer		res	1

buffer		res	8	; source/destination buffer for testing...
leftbuttonstate	res	1
PBSwitches	res	1

state	res	1
counter	res	1

	extern	InitUSB
	extern	PutEP1
	extern	GetEP1
	extern	ServiceUSBInt
	extern	CheckSleep
	extern	RemoteWakeup

STARTUP	code
	pagesel	main
	goto	main
	nop
InterruptServiceVector
	movwf	W_save		; save W
	movf	STATUS,W
	clrf	STATUS		; force to page 0
	movwf	Status_save	; save STATUS
	movf	PCLATH,w
	movwf	PCLATH_save	; save PCLATH
	movf	FSR,w
	movwf	FSR_save	; save FSR

; *************************************************************
; Interrupt Service Routine
; First we step through several stages, attempting to identify the source 
; of the interrupt.
; ******************************************************************

Process_ISR
; Step 1, what triggered the interrupt?

	btfsc	INTCON,T0IF	; Timer 0
	nop
	btfss	INTCON,RBIF	; Port B
	goto	TEST_INTCON
	call	PortBChange

TEST_INTCON
	btfsc	INTCON,INTF	; External Interrupt
	nop
	banksel	PIR1
	pagesel	ServiceUSBInt
	btfss	PIR1,USBIF
	goto	TEST_PIR1
	call	ServiceUSBInt

TEST_PIR1
	btfsc	PIR1,ADIF	; AD Done?
	nop
	btfsc	PIR1,RCIF
	nop
	btfsc	PIR1,TXIF
	nop
	btfsc	PIR1,CCP1IF
	nop
	btfsc	PIR1,TMR2IF
	nop
	btfsc	PIR1,TMR1IF
	nop
	btfsc	PIR2,CCP2IF
	nop

; ******************************************************************
; End ISR, restore context and return to the main program
; ******************************************************************
EndISR
	clrf	STATUS		; select bank 0
	movf	FSR_save,w	; restore the FSR
	movwf	FSR
	movf	PCLATH_save,w	; restore PCLATH
	movwf	PCLATH
	movf	Status_save,w	; restore Status
	movwf	STATUS
	swapf	W_save,f	; restore W without corrupting STATUS
	swapf	W_save,w
	retfie


; **********************************************************************
; PortB Change Interrupt Handler, calls RemoteWakeup to perform wakeup
; **********************************************************************
PortBChange
	clrf	STATUS
	movf	PORTB,w
	bcf	INTCON,RBIF
	pagesel RemoteWakeup
	call RemoteWakeup
	return

	code
; ******************************************************************
; test program that sets up the buffers and calls the ISR for processing.
; 	
; ******************************************************************
main
#ifdef	SHOW_ENUM_STATUS
	banksel	TRISB
	clrf	TRISB		; Set all port B to output
	banksel	PORTB
	clrf	PORTB		; clear all port B outputs
#endif
	movlw	.30		; delay 16 uS to wait for USB to reset
	movwf	W_save		; SIE before initializing registers
	decfsz	W_save,f	; inner is merely a convienient register
	goto	$-1		; to use for the delay counter.

; ******************************************************************
; Sets the probe control register to output the UCTRL register and
; USBDPRAM databus onto the probepins.  
; ******************************************************************
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	bcf	OPTION_REG,NOT_RBPU  ; enable portb weak pull-up to detected change in PORTB

	pagesel	InitUSB
	call	InitUSB
	pagesel	main
	ConfiguredUSB		; wait here until the enumeration process is complete

idleloop
	bankisel  buffer
	movlw	buffer		; point IRP+FSR to destination buffer
	movwf	FSR
	pagesel	GetEP1		; ensure PCLATH bits are set
	call	GetEP1	
	pagesel	idleloop
	btfss	STATUS,C	; was any data copied?
	goto	idleloop	; no: keep trying

; yes: Message received.  Increment the bytes and send them back.
bumpit
	banksel	buffer
	incf	buffer,f
	incf	buffer+1,f

	bankisel  buffer	; Point IRP+FSR to destination buffer
	movlw	buffer
	movwf	FSR
	movlw	2		; sending 2 bytes
	pagesel	PutEP1		
	call	PutEP1		
	pagesel	bumpit
	btfsc	STATUS,C	; was the put successful?
	goto	bumpit		; no: keep trying
	goto	idleloop	; yes: wait for next buffer


; *********************************************************************
; SetConfiguration
; 
; This function is called when the host issues a Set Configuration 
; command.  The housekeeping within USB is handled within the CH9 commands
; This function should be filled in to give meaning to the command within
; the application.
;
; SetConfiguration is called from within the ISR so this function should
; be kept as short as possible.
; *********************************************************************
SetConfiguration
	global	SetConfiguration
	return

; *********************************************************************
; Vendor Specific calls
; control is transferred here when bmRequestType bits 5 & 6 = 10 indicating
; the request is a vendor specific request.  This function then would 
; interpret the bRequest field to determine what action is required.
; The end of each vendor specific command should be terminated with a 
; return.  
; *********************************************************************
CheckVendor	
	global	CheckVendor
	return	; *** remove this line and uncomment out the remainder	

	end

